/*
  "EEPROMdataStorage"
  Library supporting storage of different data formats in Arduino EEPROM
  V1.1 / Copyright (C) 2017, T.Rode (DL1DUZ)

  Permission is granted to use, copy, modify, and distribute this software
  and documentation for non-commercial purposes.
*/

//################################################################################################################
//includes
//################################################################################################################

#include <EEPROMdataStorage.h>

/*################################################################################################################
//declarations
//################################################################################################################

//################################################################################################################
//private functions
//################################################################################################################

transfers data between EEPROM and RAM / returns the next EEPROM address
"address" -> start address in EEPROM
"ram" -> pointer to ram (source/target of data transfer)
"size" -> size of the data (e.g. 4 for variables of the type long)
"elements" -> number of elements (e.g. 5 for an array of 5)
"write" -> specifies if data will be written or read to/from EEPROM
*/
uint16_t EEPROMdataStorageClass::dataTransfer(uint16_t *address, uint8_t *ram, uint8_t *size, uint16_t *elements,
                                              uint8_t write) {
  uint32_t scope = *size * *elements;
  for(uint32_t i=0; i<scope; i++) {
    if(write) {
      EEPROM.update(*address + i, *(ram + i));
    }
    else {
      *(ram + i) = EEPROM.read(*address + i);
    }
  }

  return *address + scope;
}

/*################################################################################################################
//public functions
//################################################################################################################

reads data from the EEPROM / returns the next EEPROM address
"address" -> start address in EEPROM
"target" -> target for data transfer (e.g. single variable, array)
"size" -> size of the data to be read (e.g. 4 for variables of the type long)
"elements" -> number of elements to be read
*/
uint16_t EEPROMdataStorageClass::dataFromEEPROM(uint16_t address, void *target, uint8_t size, uint16_t elements) {
  return dataTransfer(&address, (uint8_t *) target, &size, &elements, 0);
}

/*################################################################################################################
writes data to the EEPROM / returns the next free EEPROM address
"address" -> start address in EEPROM
"source" -> source for data transfer (e.g. single variable, array)
"size" -> size of the data to be written (e.g. 4 for variables of the type long)
"elements" -> number of elements to be written
*/
uint16_t EEPROMdataStorageClass::dataToEEPROM(uint16_t address, const void *source, uint8_t size, uint16_t elements) {
  return dataTransfer(&address, (uint8_t *) source, &size, &elements, 1);
}

EEPROMdataStorageClass EEPROMDS;
